#include "realtime_adv_wrapper.h"

#include <string.h>
#include <time.h>
#include <sys/time.h>
#include <errno.h>

namespace rtwrapper {

static void MicroSecSleep(int64_t us) {
    struct timespec tm;
    tm.tv_sec = us / (1000*1000);
    tm.tv_nsec = (us - tm.tv_sec*1000*1000)*1000;
    nanosleep(&tm, NULL);
}

bool RTTask::Sleep(int64_t us) {
    RTIME_W ns = us;
    ns *= 1000L;
    return 0 == rt_task_sleep_w(ns);
}

RTTask::RTTask() : _ok(false), _start(false), _errno(0) {
    memset(&this->_task, 0, sizeof(this->_task));
}

RTTask::~RTTask() {
}

bool RTTask::Create(const char *name, int prio, int cpus[]) {
    if (this->_ok)
        return false;
    if (name == NULL)
        return false;
    _errno = rt_task_create_w(&this->_task, name, 0, prio, T_JOINABLE_W, cpus);
    if ( 0 !=_errno )
        return false;
    this->_ok = true;
    _errno = rt_task_start_w(&this->_task, &_Process, this);
    if (0 != _errno ) {
        Kill();
        return false;
    }
    this->_start = true;
    return true;
}

bool RTTask::Close() {
    if (!this->_ok)
        return false;
    Kill();
    return true;
}

bool RTTask::Join() {
    if (!this->_ok)
        return false;
    _errno = rt_task_join_w(&this->_task);
    if ( 0 != _errno ) {
        return false;
    }
    this->_ok = false;
    memset(&this->_task, 0, sizeof(this->_task));
    return true;
}

bool RTTask::Kill() {
    if (!this->_ok)
        return false;
    _errno = rt_task_delete_w(&this->_task);
    if (0 != _errno ) {
        return false;
    }
    return Join();
}

bool RTTask::IsValid() const {
    return this->_ok;
}

void RTTask::_Process(void *param) {
    RTTask *task = (RTTask*)param;
    task->Process();
}

// ---Actor---
RTActor::RTActor(const char *name, int prio, int cpu) : RTTask(), _shutdown(false), _done(false)
                                                        , _prio(prio)
{
    memset(_name, 0, sizeof(_name));
    memcpy(_name, name, sizeof(_name)-1);

    for ( int i=0;i<MAX_CPU;i++ )
    {
        _cpus[i] = -1;
    }
    _cpus[0] = cpu;
}


RTActor::RTActor(const char *name, int prio, int cpus[]) : RTTask(), _shutdown(false), _done(false)
                                                         , _prio(prio)
{
    memset(_name, 0, sizeof(_name));
    memcpy(_name, name, sizeof(_name)-1);

    int i=0;
    for ( ;i<MAX_CPU;i++ )
    {
        if ( 0>cpus[i] )
        {
            break;
        }
        _cpus[i] = cpus[i];
    }
    _cpus[i] = -1;
}

RTActor::RTActor() : RTTask(), _shutdown(false), _done(false), _prio(0) {
    memset(_name, 0, sizeof(_name));
    for ( int i=0;i<MAX_CPU;i++ )
    {
        _cpus[i] = -1;
    }
}

RTActor::~RTActor() {
}

bool RTActor::Start() {
    return Create(_name, _prio, _cpus);
}

void RTActor::Shutdown() {
    _shutdown = true;
    if ( IsStart() )
    {
        int _try = 600*100;
        while(!_done&&_try--)
        {
            MicroSecSleep(10000);
        }
    }

    Close();
}

void RTActor::Process() {
    while ( !_shutdown ) {
        Run();
    }
    _done = true;
}

} // namespace
